@@ -7,6 +7,7 @@ require 'tmpdir'
 require 'chake/version'
 require 'chake/node'
 require 'chake/readline'
+require 'chake/tmpdir'
 
 nodes_file = ENV['CHAKE_NODES'] || 'nodes.yaml'
 nodes_directory = ENV['CHAKE_NODES_D'] || 'nodes.d'
@@ -15,7 +16,7 @@ Dir.glob(File.join(nodes_directory, '*.yaml')).sort.each do |f|
   node_data.merge!(YAML.load_file(f))
 end
 $nodes = node_data.map { |node,data| Chake::Node.new(node, data) }.reject(&:skip?).uniq(&:hostname)
-$chake_tmpdir = ENV.fetch('CHAKE_TMPDIR', 'tmp/chake')
+$chake_tmpdir = Chake.tmpdir
 
 desc "Initializes current directory with sample structure"
 task :init do
@@ -206,33 +207,80 @@ $nodes.each do |node|
     end
   end
 
+  converge_dependencies = [:converge_common, "bootstrap:#{hostname}", "upload:#{hostname}"]
+
   desc "converge #{hostname}"
-  task "converge:#{hostname}" => [:converge_common, "bootstrap:#{hostname}", "upload:#{hostname}"] do
+  task "converge:#{hostname}" => converge_dependencies do
     chef_logging = Rake.application.options.silent && '-l fatal' || ''
     node.run_as_root "chef-solo -c #{node.path}/config.rb #{chef_logging} -j #{node.path}/#{$chake_tmpdir}/#{hostname}.json"
   end
 
+  desc 'apply <recipe> on #{hostname}'
+  task "apply:#{hostname}", [:recipe] => [:recipe_input] do |task, args|
+    chef_logging = Rake.application.options.silent && '-l fatal' || ''
+    node.run_as_root "chef-solo -c #{node.path}/config.rb #{chef_logging} -j #{node.path}/#{$chake_tmpdir}/#{hostname}.json --override-runlist recipe[#{$recipe_to_apply}]"
+  end
+  task "apply:#{hostname}" => converge_dependencies
+
   desc "run a command on #{hostname}"
-  task "run:#{hostname}" => 'run_input' do
-    node.run($cmd)
+  task "run:#{hostname}", [:command] => [:run_input] do
+    node.run($cmd_to_run)
   end
 
   desc "Logs in to a shell on #{hostname}"
   task "login:#{hostname}" do
     node.run_shell
   end
+
+  desc 'checks connectivity and setup on all nodes'
+  task "check:#{hostname}" do
+    node.run('sudo true')
+  end
+
 end
 
-task :run_input do
-  puts "# Enter command to run (use arrow keys for history):"
-  $cmd = ENV['CMD'] || Chake::Readline.readline
-  if !$cmd || $cmd.strip == ''
+task :run_input, :command do |task,args|
+  $cmd_to_run = args[:command]
+  if !$cmd_to_run
+    puts "# Enter command to run (use arrow keys for history):"
+    $cmd_to_run = Chake::Readline::Commands.readline
+  end
+  if !$cmd_to_run || $cmd_to_run.strip == ''
     puts
     puts "I: no command provided, operation aborted."
     exit(1)
   end
 end
 
+task :recipe_input, :recipe do |task,args|
+  $recipe_to_apply = args[:recipe]
+
+  if !$recipe_to_apply
+    recipes = Dir['**/*/recipes/*.rb'].map do |f|
+      f =~ %r{(.*/)?(.*)/recipes/(.*).rb$}
+      cookbook = $2
+      recipe = $3
+      recipe = nil if recipe == 'default'
+      [cookbook,recipe].compact.join('::')
+    end.sort
+    puts 'Available recipes:'
+
+    IO.popen('column', 'w') do |column|
+      column.puts(recipes)
+    end
+
+    $recipe_to_apply = Chake::Readline::Recipes.readline
+    if !$recipe_to_apply || $recipe_to_apply.empty?
+      puts
+      puts "I: no recipe provided, operation aborted."
+      exit(1)
+    end
+    if !recipes.include?($recipe_to_apply)
+      abort "E: no such recipe: #{$recipe_to_apply}"
+    end
+  end
+end
+
 desc "upload to all nodes"
 task :upload => $nodes.map { |node| "upload:#{node.hostname}" }
 
@@ -242,7 +290,15 @@ task :bootstrap => $nodes.map { |node| "bootstrap:#{node.hostname}" }
 desc "converge all nodes (default)"
 task "converge" => $nodes.map { |node| "converge:#{node.hostname}" }
 
-desc "run a command on all nodes"
-task :run => $nodes.map { |node| "run:#{node.hostname}" }
+desc "Apply <recipe> on all nodes"
+task "apply", [:recipe] => $nodes.map { |node| "apply:#{node.hostname}" }
+
+desc "run <command> on all nodes"
+task :run, [:command] => $nodes.map { |node| "run:#{node.hostname}" }
 
 task :default => :converge
+
+desc 'checks connectivity and setup on all nodes'
+task :check => ($nodes.map { |node| "check:#{node.hostname}" }) do
+  puts "✓ all hosts OK"
+end
